package IQXML;

# ------------------------------------------------------------------------

#Copyright (C) 2005 Enrique Amig and Jess Gimnez

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License as published by the Free Software Foundation; either
#version 2.1 of the License, or (at your option) any later version.

#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public
#License along with this library; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# ------------------------------------------------------------------------

use strict;
use IO;
use IO::File;
use Data::Dumper;
use XML::Twig;
use Unicode::String;

my $GZIP = "gzip -f";

# !! watch memory leaks when using XML::Twig !!

sub write_report {
   #description _ writes a given list of segment MT scores by a given metric onto an XML report file
   #param1  _ TARGET
   #param2  _ REFERENCE
   #param3  _ METRIC name
   #param4  _ system-level scores for the given metric
   #param5  _ list of segment scores for the given metric
   #param6  _ xml report filename (output)
   #param7  _ verbosity (0/1)

   my $TGT = shift;
   my $REF = shift;
   my $METRIC = shift;
   my $SCORE = shift;
   my $M = shift;
   my $xml = shift;
   my $verbose = shift;

   if ($verbose > 1) { print STDERR "writing XML-IQREPORT <$xml>\n"; }

   my $OUT = new IO::File("> $xml");

   print $OUT "<?xml version=\"1.0\"?>\n";                  # print the XML declaration
   print $OUT "<!DOCTYPE iqmt SYSTEM \"iqmt.dtd\" []>";

   my $REPORT = new XML::Twig::Elt('IQREPORT');

   ## -- SEGMENT*
   my $N = scalar(@{$M});
   my $nSEGMENTS = 0;
   while ($nSEGMENTS < $N) {
      my $SEG = new XML::Twig::Elt('S');
      my $x = sprintf("%8.4f", $M->[$nSEGMENTS]);
      $x =~ s/^ +//g;
      $SEG->set_text($x);
      my %attseg = ('n'=>$nSEGMENTS+1);
      $SEG->set_atts(\%attseg);
      $SEG->paste('last_child', $REPORT);
      $nSEGMENTS++;
   }

   # -> STORE IQREPORT
   my $score = sprintf("%8.4f", $SCORE); $score =~ s/^ +//g;
   my %attreport = ('metric'=>$METRIC, 'target'=>$TGT, 'ref'=>$REF, 'score'=>$score);
   $REPORT->set_atts(\%attreport);
   $REPORT->set_pretty_print('record');
   $REPORT->print($OUT);

   my @SGMS = $REPORT->children;
   for (my $s = 0; $s < scalar(@SGMS); $s += 1) { $SGMS[$s]->cut_children(); }
   $REPORT->cut_children();

   $OUT->close();    

   system("$GZIP $xml");
}

sub read_report {
   #description _ reads a list of segment MT scores for a given metric
   #              from a given XML report file onto memory. (single reference)
   #param1  _ TARGET
   #param2  _ REFERENCE
   #param3  _ METRIC name
   #param4  _ xml report filename (input)
   #param5  _ IQ structure (hash ref)
   #param6  _ set of segments (topics) (list ref)
   #param7  _ level of granularity ('sys' for system; 'seg' for segment)
   #param8  _ verbosity (0/1)

   my $TGT = shift;
   my $REF = shift;
   my $METRIC = shift;
   my $xml = shift;
   my $IQ = shift;
   my $T = shift;
   my $G = shift;
   my $verbose = shift;

   if (!(-e $xml) and (-e "$xml.$METRICS::GZEXT")) { system("gunzip $xml.$METRICS::GZEXT"); }

   if (-e $xml) {   
      if ($verbose > 1) { print STDERR "reading XML-IQREPORT <$xml>\n"; }

      my $twig = XML::Twig->new( keep_encoding => 1 );
      $twig->parsefile($xml);
      my $REPORT = $twig->root;
      #my $metric = $REPORT->att('metric');
      #my $target = $REPORT->att('target');
      #my $ref = $REPORT->att('ref');

      if ($G eq 'sys') {
         my $score =  $REPORT->att('score');
         $IQ->{$METRIC}->{$TGT}->{$REF} = $score;
      }
      elsif ($G eq 'seg') {
         my @SGMS = $REPORT->children;
         for (my $s = 0; $s < scalar(@SGMS); $s += 1) {
            my $x = $SGMS[$s]->text;
            my $n = $SGMS[$s]->att('n');
            if ($T->[$n]) { $IQ->{$METRIC}->[$n]->{$TGT}->{$REF} = $x; }
         }
      }
      else { die "[ERROR] UNKNOWN granularity level <$G>!!n"; }

      $twig->dispose();

      system("$GZIP $xml");
   }
}

sub read_write_report {
   #description _ reads a list of segment MT scores for a given metric from a given XML report file
   #              and prints onto an IQ.table of similarities
   #param1  _ TARGET
   #param2  _ REFERENCE
   #param3  _ METRIC name
   #param4  _ xml report filename (output)
   #param5  _ output file handle (IQ table)
   #param6  _ verbosity (0/1)

   my $TGT = shift;
   my $REF = shift;
   my $METRIC = shift;
   my $xml = shift;
   my $IQ = shift;
   my $verbose = shift;
 
   if (!(-e $xml) and (-e "$xml.$METRICS::GZEXT")) { system("gunzip $xml.$METRICS::GZEXT"); }

   if (-e $xml) {   
      if ($verbose > 1) { print STDERR "reading XML-IQREPORT <$xml>\n"; }

      my $twig = XML::Twig->new( keep_encoding => 1 );
      $twig->parsefile($xml);
      my $REPORT = $twig->root;
      #my $metric = $REPORT->att('metric');
      #my $ref = $REPORT->att('ref');
      #my $target = $REPORT->att('target');

      my $nSEGMENTS = 0;
      my @SGMS = $REPORT->children;
      for (my $s = 0; $s < scalar(@SGMS); $s += 1) {
         my $x = $SGMS[$s]->text;
         my $n = $SGMS[$s]->att('n');
         print $IQ "$METRIC\t$n\t$TGT\t$REF\t$x\n";
         $nSEGMENTS++;
      }

      $twig->dispose();

      system("$GZIP $xml");
   }
}

sub BLEU_f_create_mteval_doc {
    #description _ creation of an XML Twig from a "sentence-per-line" format corpus
    #param1  _ input file
    #param2  _ output file
    #param3  _ verbose (0/1)

    my $input = shift;
    my $output = shift;
    my $verbose = shift;

    if ($verbose > 1) { print STDERR "OPENING <$input> for BLEU xml-parsing...\n"; }

    my $IN = new IO::File("< $input") or die "Couldn't open input file: $input\n";
    my $OUT = new IO::File("> $output");

    my $nSEGMENTS = 1;

    my $DOC = new XML::Twig::Elt('DOC');
    ## -- DOCUMENTS*
    while (defined (my $line = <$IN>)) {
       chomp($line);
       $line =~ s/ +$//g;
       if ($line eq "") { $line = "***EMPTY***"; }
       my $SEG = new XML::Twig::Elt('segment');
       $SEG->set_text($line);
       $SEG->paste('last_child', $DOC);
       $nSEGMENTS++;
    }

    # -> STORE DOCUMENT
    my %att = ('doc_ID'=>"dummydoc", 'sys_ID'=>"dummysys");
    $DOC->set_atts(\%att);
    $DOC->set_pretty_print('record');
    $DOC->print($OUT);
    
    $IN->close();    
    $OUT->close();    

    my @SGMS = $DOC->children;
    for (my $s = 0; $s < scalar(@SGMS); $s += 1) { $SGMS[$s]->cut_children(); }
    $DOC->cut_children();
}

sub NIST_f_create_mteval_doc {
    #description _ creation of an XML Twig from a "sentence-per-line" format corpus
    #param1  _ input file
    #param2  _ output file
    #param3  _ type (0:srcset :: 1:tstset :: 2:refset)
    #param4  _ verbose (0/1)

    my $input = shift;
    my $output = shift;
    my $type = shift;
    my $verbose = shift;

    if ($verbose > 1) { print STDERR "OPENING <$input> for NIST xml-parsing [type = ", ($type == 0)? "SRC" : ($type == 1)? "TST" : "REF", "]...\n"; }

    my $IN = new IO::File("< $input") or die "Couldn't open input file: $input\n";
    my $OUT = new IO::File("> $output");

    my $nSEGMENTS = 1;

    my $set;
    my $sysid;
    if ($type == 0) { $set = 'srcset'; $sysid = "dummysysSRC"; }
    elsif ($type == 1) { $set = 'tstset'; $sysid = "dummysysTST"; }
    elsif ($type == 2) { $set = 'refset'; $sysid = "dummysysREF"; }

    my $SET = new XML::Twig::Elt($set);
    my $DOC = new XML::Twig::Elt('DOC');
    ## -- DOCUMENTS*
    while (defined (my $line = <$IN>)) {
       chomp($line);
       $line =~ s/ +$//g;
       if ($line eq "") { $line = "***EMPTY***"; }
       my $SEG = new XML::Twig::Elt('seg');
       #my %segatt = ('id'=>$nSEGMENTS + 0);
       #$SEG->set_atts(\%segatt);
       $SEG->set_id($nSEGMENTS);
       $SEG->set_text($line);
       $SEG->paste('last_child', $DOC);
       $nSEGMENTS++;
    }

    # -> STORE DOCUMENT
    my %docatt = ('docid'=>"dummydoc", 'sysid'=>"$sysid");
    $DOC->set_atts(\%docatt);
    $DOC->paste('last_child', $SET);

    my %setatt = ('setid'=>"dummyset", 'srclang'=>"dummylang", 'trglang'=>"dummylang");
    $SET->set_atts(\%setatt);
    $SET->set_pretty_print('record');
    $SET->print($OUT);

    $IN->close();    
    $OUT->close();
 
    my @DOCS = $SET->children;
    for (my $d = 0; $d < scalar(@DOCS); $d += 1) {
       my @SGMS = $DOCS[$d]->children;
       for (my $s = 0; $s < scalar(@SGMS); $s += 1) { $SGMS[$s]->cut_children(); }
       $DOCS[$d]->cut_children();
    }
    $SET->cut_children();
}

sub NIST_f_create_mteval_multidoc {
    #description _ creation of an XML Twig from a "sentence-per-line" format corpus
    #param1  _ input file(s) (hash reference)
    #param2  _ output file
    #param3  _ type (0:srcset :: 1:tstset :: 2:refset)
    #param4  _ verbose (0/1)

    my $Hinput = shift;
    my $output = shift;
    my $type = shift;
    my $verbose = shift;

    if ($verbose > 1) { print STDERR "OPENING [", join(" ", sort keys %{$Hinput}), "] for NIST* xml-parsing..."; }

    my $OUT = new IO::File("> $output");

    my $set;
    my $sysid;
    if ($type == 0) { $set = 'srcset'; $sysid = "dummysysSRC"; }
    elsif ($type == 1) { $set = 'tstset'; $sysid = "dummysysTST"; }
    elsif ($type == 2) { $set = 'refset'; $sysid = "dummysysREF"; }

    my $SET = new XML::Twig::Elt($set);

    foreach my $sysname (sort METRICS::criterion keys %{$Hinput}) {
       my $input = $Hinput->{$sysname};
       my $IN = new IO::File("< $input") or die "Couldn't open input file: $input\n";
       my $DOC = new XML::Twig::Elt('DOC');
       my $nSEGMENTS = 1;

       ## -- DOCUMENTS*
       while (defined (my $line = <$IN>)) {
          chomp($line);
          $line =~ s/ +$//g;
          if ($line eq "") { $line = "***EMPTY***"; }
          my $SEG = new XML::Twig::Elt('seg');
          #my %segatt = ('id'=>$nSEGMENTS + 0);
          #$SEG->set_atts(\%segatt);
          $SEG->set_id($nSEGMENTS);
          $SEG->set_text($line);
          $SEG->paste('last_child', $DOC);
          $nSEGMENTS++;
       }

       # -> STORE DOCUMENT
       my %docatt = ('docid'=>"dummydoc", 'sysid'=>"$sysname");
       $DOC->set_atts(\%docatt);
       $DOC->paste('last_child', $SET);
       $IN->close();     
    }

    my %setatt = ('setid'=>"dummyset", 'srclang'=>"dummylang", 'trglang'=>"dummylang");
    $SET->set_atts(\%setatt);
    $SET->set_pretty_print('record');
    $SET->print($OUT);

    $OUT->close();    

    my @DOCS = $SET->children;
    for (my $d = 0; $d < scalar(@DOCS); $d += 1) {
       my @SGMS = $DOCS[$d]->children;
       for (my $s = 0; $s < scalar(@SGMS); $s += 1) { $SGMS[$s]->cut_children(); }
       $DOCS[$d]->cut_children();
    }
    $SET->cut_children();
}

sub GTM_f_create_mteval_doc {
    #description _ creation of an XML Twig from a "sentence-per-line" format corpus
    #param1  _ input file
    #param2  _ output file
    #param3  _ verbose (0/1)

    my $input = shift;
    my $output = shift;
    my $verbose = shift;

    if ($verbose > 1) { print STDERR "OPENING <$input> for GTM xml-parsing...\n"; }

    my $IN = new IO::File("< $input") or die "Couldn't open input file: $input\n";
    my $OUT = new IO::File("> $output");

    my $nSEGMENTS = 1;

    my $DOC = new XML::Twig::Elt('doc');
    ## -- DOCUMENTS*
    while (defined (my $line = <$IN>)) {
       chomp($line);
       $line =~ s/ +$//g;
       if ($line eq "") { $line = "***EMPTY***"; }
       my $SEG = new XML::Twig::Elt('seg');
       $SEG->set_id($nSEGMENTS);
       $SEG->set_text($line);
       $SEG->paste('last_child', $DOC);
       $nSEGMENTS++;
    }

    # -> STORE DOCUMENT
    my %att = ('doc_id'=>"dummydoc", 'sysid'=>"dummysys");
    $DOC->set_atts(\%att);
    $DOC->set_pretty_print('record');
    $DOC->print($OUT);

    $IN->close();    
    $OUT->close();    

    my @SGMS = $DOC->children;
    for (my $s = 0; $s < scalar(@SGMS); $s += 1) { $SGMS[$s]->cut_children(); }
    $DOC->cut_children();
}

1;
